#include "timer.h"

timer_callback event_handler_bank [MAX_TIMER_INSTANCES] [MAX_CHANNEL_INSTANCES];

uint32_t event_storage[MAX_TIMER_INSTANCES][MAX_CHANNEL_INSTANCES];



void TIMER1_IRQHandler(void)
{

    NRF_TIMER1->EVENTS_COMPARE[0] = 0;
    volatile uint32_t dummy = NRF_TIMER1->EVENTS_COMPARE[0];
    (void)dummy;
    event_handler_bank[0][0].event_handler(); //calls test_handler()
}

void TIMER2_IRQHandler(void)
{

  NRF_TIMER2->EVENTS_COMPARE[0] = 0;
  volatile uint32_t dummy = NRF_TIMER2->EVENTS_COMPARE[0];
  (void)dummy;
  event_handler_bank[1][0].event_handler(); //calls test_handler2()
}



void enable_timer_instance(NRF_TIMER_Type *instance)
{
  switch((uint32_t)instance)
  {
    case (uint32_t)NRF_TIMER1:
    {
      NVIC_EnableIRQ(TIMER1_IRQn);
      break;
    }
    case (uint32_t)NRF_TIMER2:
    {
      NVIC_EnableIRQ(TIMER2_IRQn);
      break;
    }
    case (uint32_t)NRF_TIMER3:
    {
      NVIC_EnableIRQ(TIMER3_IRQn);
      break;
    }
    case (uint32_t)NRF_TIMER4:
    {
      NVIC_EnableIRQ(TIMER4_IRQn);
      break;
    }
  }
  
}

void set_timer_shorts(NRF_TIMER_Type *instance, uint32_t shorts_pos, uint32_t sel_short)
{
  if(sel_short)
  {
    instance->SHORTS |= (1 << shorts_pos);
  }
  else
  {
    instance ->SHORTS &= ~(1 << shorts_pos);
  }
}

void set_timer_interrupt(NRF_TIMER_Type *instance, uint32_t int_pos, uint8_t enable)
{
  if(enable)
  {
    instance->INTENSET |= (1 << int_pos);
  }
  else
  {
    instance->INTENCLR |= (1 << int_pos);
  }
}

void set_timer_mode(NRF_TIMER_Type *instance, uint32_t mode)
{
  instance->MODE = mode;
}

void set_timer_bitmode(NRF_TIMER_Type *instance, uint32_t mode)
{
  instance->TASKS_STOP = 1;
  instance->BITMODE = mode;
}

void set_timer_prescaler(NRF_TIMER_Type *instance, uint32_t prescaler)
{
  instance->TASKS_STOP = 1;
  instance->PRESCALER = prescaler;
}

void set_timer_compare(NRF_TIMER_Type *instance, uint32_t channel, uint32_t compare_val, timer_event_handler_t event_handler)
{
  instance->CC[channel] = compare_val;
  switch((uint32_t)instance)
  {
    case (uint32_t)NRF_TIMER1:
    {
      event_handler_bank[0][channel].event_compare = instance->EVENTS_COMPARE[channel];
      event_handler_bank[0][channel].event_handler = event_handler;
      break;
    }
    case (uint32_t)NRF_TIMER2:
    {
      event_handler_bank[1][channel].event_compare = instance->EVENTS_COMPARE[channel];
      event_handler_bank[1][channel].event_handler = event_handler;
      break;
    }
    case (uint32_t)NRF_TIMER3:
    {
      event_handler_bank[2][channel].event_compare = instance->EVENTS_COMPARE[channel];
      event_handler_bank[2][channel].event_handler = event_handler;
      break;
    }
    case (uint32_t)NRF_TIMER4:
    {
      event_handler_bank[3][channel].event_compare = instance->EVENTS_COMPARE[channel];
      event_handler_bank[3][channel].event_handler = event_handler;
      break;
    }
  }
}

void stop_timer(NRF_TIMER_Type *instance)
{
  instance->TASKS_STOP = 1;
}

void start_timer(NRF_TIMER_Type *instance)
{
  instance->TASKS_START = 1;
}

void clear_timer(NRF_TIMER_Type *instance)
{
  instance->TASKS_CLEAR = 1;
}

void capture_timer(NRF_TIMER_Type *instance, uint32_t channel)
{
  instance->TASKS_CAPTURE[channel] = 1;
}