/**
 * Copyright (c) 2014 - 2019, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/** @file
*
* @defgroup gpiote_example_main main.c
* @{
* @ingroup nrf_gpiote_example
* @brief GPIOTE Example Application main file.
*
* This file contains the source code for a sample application using GPIOTE.
*/

#include <stdbool.h>
#include <stdint.h>
#include "nrf.h"
#include "gpio.h"
#include "timer.h"
/**
 * @brief Function for application main entry.
 */

volatile uint8_t t1 = 0;
volatile uint8_t t2 = 0;


void test_handler(void)
{
  if(t1)
  {
    gpio_pin_clear(NRF_P0,17);
    t1 = 0;
  }
  else
  {
    gpio_pin_set(NRF_P0,17);
    t1 = 1;
  }
}

void test_handler2(void)
{
  if(t2)
  {
    gpio_pin_clear(NRF_P0,18);
    t2 = 0;
  }
  else
  {
    gpio_pin_set(NRF_P0,18);
    t2 = 1;
  }

}

int main(void)
{
    gpio_config config;
    config.port = NRF_P0;
    config.pin  = 17;
    config.dir = DIR_OUTPUT;
    config.input_buffer = INPUT_DISC;
    config.pull = PULL_NONE;
    config.drive = DRIVE_S0S1;
    config.sense = SENSE_DIS;
    gpio_config_pin(&config);

//    gpio_config config;
    config.port = NRF_P0;
    config.pin  = 18;
    config.dir = DIR_OUTPUT;
    config.input_buffer = INPUT_DISC;
    config.pull = PULL_NONE;
    config.drive = DRIVE_S0S1;
    config.sense = SENSE_DIS;
    gpio_config_pin(&config);



    enable_timer_instance(NRF_TIMER1);
    set_timer_shorts(NRF_TIMER1, TIMER_SHORT_CMP0_CLR_POS,TIMER_SHORT_CMP0_CLR_ENABLE);
    set_timer_mode(NRF_TIMER1, TIMER_MODE_TIMER);
    set_timer_prescaler(NRF_TIMER1, TIMER_PRESCALER_9);
    set_timer_bitmode(NRF_TIMER1, TIMER_BITMODE_16);
    set_timer_interrupt(NRF_TIMER1, TIMER_INT_CMP0_POS, 1);
    set_timer_compare(NRF_TIMER1, 0, 50000, test_handler);

    enable_timer_instance(NRF_TIMER2);
    set_timer_shorts(NRF_TIMER2, TIMER_SHORT_CMP0_CLR_POS,TIMER_SHORT_CMP0_CLR_ENABLE);
    set_timer_mode(NRF_TIMER2, TIMER_MODE_TIMER);
    set_timer_prescaler(NRF_TIMER2, TIMER_PRESCALER_9);
    set_timer_bitmode(NRF_TIMER2, TIMER_BITMODE_24);
    set_timer_interrupt(NRF_TIMER2, TIMER_INT_CMP0_POS, 1);
    set_timer_compare(NRF_TIMER2, 0, 25000, test_handler2);


    start_timer(NRF_TIMER2);
    start_timer(NRF_TIMER1);


    while (true)
    {
    }
}


/** @} */
